# mill.py, Markdown interface for llama.cpp
# Copyright (C) 2024 unworriedsafari <unworriedsafari@tilde.club>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

example = """
## Example

Contents of `hello.md`:

    ## Variables

    ```mill-llm
    --model
    mixtral-8x7b-instruct-v0.1.Q5_0.gguf
    ```
    
    ```mill-llm
    --ctx-size
    0
    ```
    
    ```mill-llm
    --keep
    -1
    ```
    
    ```mill
    message template
    

    Me:
    
    > [INST]  [/INST]
    
    Bot:
    
    >
    ```
    
    ```mill
    prompt indent
    >
    ```
    
    ## Chat
    
    ```mill
    prompt start
    ```
    
    Me:
    
    > [INST] Hello, how are you? [/INST]
    
    Bot:
    
    >

Command:

```bash
export MILL_LLAMACPP_MAIN=path/to/llama.cpp/main /path/to/mill_cli.py <hello.md
```

Output:

    ## Variables

    ```mill-llm
    --model
    mixtral-8x7b-instruct-v0.1.Q5_0.gguf
    ```
    
    ```mill-llm
    --ctx-size
    0
    ```
    
    ```mill-llm
    --keep
    -1
    ```
    
    ```mill
    message template


    Me:

    > [INST]  [/INST]

    Bot:
    
    >
    ```
    
    ```mill
    prompt indent
    >
    ```
    
    ## Chat
    
    ```mill
    prompt start
    ```
    
    Me:
    
    > [INST] Hello, how are you? [/INST]
    
    Bot:
    
    > Hello! I'm just a computer program, so I don't have feelings, but I'm here to help you with any questions you have to the best of my ability. Is there something specific you would like to know or talk about?</s>
    
    Me:
    
    > [INST]  [/INST]
    
    Bot:
    
    >
"""


runnable_example = """
## Runnable example

```mill-llm
--model
mixtral-8x7b-instruct-v0.1.Q5_0.gguf
```

```mill-llm
--ctx-size
0
```

```mill-llm
--keep
-1
```

```mill
message template


Me:

> [INST]  [/INST]

Bot:

>
```

```mill
prompt indent
>
```

You can pipe this README to `mill.py`.

The prompt is empty, so `mill.py` will respond by adding the message
template. Also in this case, since we didn't specify the prompt start,
`mill.py` will include all prompt indent blocks.

When the prompt is not empty, the prompt is sent to the LLM and the
generated text is appended. Every newline output by the LLM introduces
a prompt indent. Finally, the message template is added.
"""
